"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
  * Plans
  *
  * @comment The plans API primarily provides functionality for a business to create, get and cancel subscription plans.
  * A subscription plan is required in order to link a user's commitment with static on-chain payment plan which can't
  * be manipulated (with the exception of a title and description/JSON metadata).
  *
  * @path eightEx.plans
  *
*/
var PlanAPI = /** @class */ (function () {
    function PlanAPI(web3, addressBook, volumeSubscriptionWrapper) {
        this.web3 = web3;
        this.addressBook = addressBook;
        this.volumeSubscriptionWrapper = volumeSubscriptionWrapper;
    }
    /**
     * Create plan
     *
     * A subscription plan is needed in order to provide an instance for a consumer to subscribe to.
     *
     * @param owner         Owner of the subscription.
     * @param token         Token to receive payments in.
     * @param identifier    External identifier to use to retrieve subscribers.
     * @param interval      Interval, in seconds, to charge a user.
     * @param amount        Amount to charge a user.
     * @param fee           Amount to set as the processing fee.
     * @param name          Your organisation/name (eg 'Netflix', 'SaaS dApp'). Shown to user.
     * @param description   Description for your plan (eg 'Premium Plan'). Shown to user.
     * @param imageUrl      Logo for your busines/plan. Shown to user.
     * @param metaData      Any extra data you'd like to store on-chain (JSON format).
     * @param txData        Provide signer, gas and gasPrice information (optional).
     *
     * @returns             Unique identifying hash of the plan.
     * @priority            1
     *
    */
    PlanAPI.prototype.create = function (owner, identifier, interval, amount, fee, name, description, imageUrl, metaData, txData) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.volumeSubscriptionWrapper.createPlan(owner, this.addressBook.transactingTokenAddress || '', identifier, interval, amount, fee, name, description, imageUrl, metaData, txData)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Get plan
     *
     * Retrieve the details of a subscription plan.
     *
     * @param planHash      Plan hash returned upon creating a plan.
     *
     * @response
     * ```
     * {
     *    owner: '0xbn38s...',
     *    tokenAddress: '0xfns83c...',
     *    identifier: 'com.your.plan.identifier',
     *    interval: 30,
     *    amount: 1000000000000000000, // Most token use 18 decimal places so this large amount translates to 10 tokens
     *    fee: 10000000000000000, // Similarly, this is actually 1/10 of a token
     *    data: '',
     *    name: 'Netflix',
     *    description: 'Premium plan',
     *    imageUrl: 'https://netflix.com/logo,
     *    terminationDate: 155324929, // (epoch in seconds),
     *    planHash: '0xd834n2k...'
     * }
     * ```
     *
     * @returns             A plan object
     * @priority            2
     *
    */
    PlanAPI.prototype.get = function (planHash) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.volumeSubscriptionWrapper.getPlan(planHash)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Get all plans
     *
     * Find all the subscription plans you've created.
     *
     * @param owner   The user who you'd like to get plans for.
     *
     * @returns       An array of Plan objects.
     * @priority      3
     *
     */
    PlanAPI.prototype.getAllFor = function (owner) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.volumeSubscriptionWrapper.getPlans(owner)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Get subscribers
     *
     * Get all the subscribers of a subscription plan.
     *
     * @param planHash    Plan hash returned upon creating a plan.
     *
     * @returns           An array of Subscription objects.
     * @priority          4
     *
     */
    PlanAPI.prototype.getSubscribers = function (planHash) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.volumeSubscriptionWrapper.getSubscriptionsByPlan(planHash)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Cancel plan
     *
     * Cancel a subscription plan that you've offered to your subscribers.
     *
     * @param planHash     Plan hash returned upon creating a plan.
     * @param txData       Provide signer, gas and gasPrice information (optional).
     *
     * @response
     * ```
     * [ 0x58e5a0fc7fbc849eddc100d44e86276168a8c7baaa5604e44ba6f5eb8ba1b7eb ]
     * ```
     *
     * @returns            Hash of the transaction upon completion.
     * @priority           5
     *
     */
    PlanAPI.prototype.cancel = function (planHash, txData) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.volumeSubscriptionWrapper.terminatePlan(planHash, txData)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    return PlanAPI;
}());
exports.default = PlanAPI;
//# sourceMappingURL=plan_api.js.map