"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var constants_1 = require("../constants");
/**
 * Subscriptions
 *
 * @comment The subscriptions api provides all the functionality for consumers to subscribe to 8x. A user must first subscribe to a subscription,
 * and then they must activate it. Payment is only taken on activation. First time 8x users are required to give pre-authorisation before
 * they can subscribe however.
 *
 * @path eightEx.subscriptions
 *
*/
var SubscriptionsAPI = /** @class */ (function () {
    function SubscriptionsAPI(web3, addressBook, volumeSubscriptionWrapper, executorWrapper, tokenWrapper) {
        this.web3 = web3;
        this.addressBook = addressBook;
        this.volumeSubscriptionWrapper = volumeSubscriptionWrapper;
        this.executorWrapper = executorWrapper;
        this.tokenWrapper = tokenWrapper;
    }
    /**
     * Check pre-authorisation
     *
     * In order for 8x to take tokens directly from a user's wallet, it needs authorisation from a user
     * to do so. This permission is given through the ERC20 approve function. Through this
     * functionality you can check whether a user has given approval to the Transfer Proxy (contract
     * that can take tokens from a user directly).
     *
     * @param owner     The user to check whether allowance has been given or not.
     *
     * @response
     * ```
     * [ true ]
     * ```
     *
     * @returns         A boolean of whether allowance has been given or not.
     * @priority        1
     *
    */
    SubscriptionsAPI.prototype.hasGivenAuthorisation = function (owner) {
        return __awaiter(this, void 0, void 0, function () {
            var allowance;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.tokenWrapper.allowance(this.addressBook.transactingTokenAddress || '', owner, this.addressBook.transferProxyAddress || '')];
                    case 1:
                        allowance = _a.sent();
                        return [2 /*return*/, allowance.toNumber() == constants_1.UNLIMITED_ALLOWANCE.toNumber()];
                }
            });
        });
    };
    /**
     * Give pre-authorisation
     *
     * Grant 8x the pre-authorisation to take tokens from a user's wallet directly. The
     * pre-authorisation is given to the Transfer Proxy contract.
     *
     * @param txData    Provide signer, gas and gasPrice information (optional).
     * @priority        2
     *
    */
    SubscriptionsAPI.prototype.giveAuthorisation = function (txData) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.tokenWrapper.giveApproval(this.addressBook.transactingTokenAddress || '', this.addressBook.transferProxyAddress || '', constants_1.UNLIMITED_ALLOWANCE, txData)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    //   /**
    //    * Subscribe and Activate
    //    * 
    //    * This function allows you to subscribe and activate a subscription via a single Ethereum transaction. In case you're
    //    * unfamiliar with the difference between subscribing and activating, subscribing is simply an agreement to pay and
    //    * activating is making the first payment and starting the flow of money.
    //    * 
    //    * @param planHash      Unique identifying hash of the plan.
    //    * @param metaData      Any extra JSON data you'd like to pass/store on-chain (optional).
    //    * @param txData        Provide signer, gas and gasPrice information (optional).
    //    * 
    //    * @returns             Unique subscription hash.
    //    * @priority            3
    //   */
    //  public async subscribeAndActivate(
    //   planHash: Bytes32,
    //   metaData: JSON | null,
    //   txData?: TxData
    // ): Promise<Bytes32> {
    //   return await this.volumeSubscriptionWrapper.subscribeAndActivate(
    //     planHash,
    //     metaData,
    //     txData
    //   );
    // }
    /**
     * Subscribe
     *
     * Subscribing links the user to the subscription plan. It does not take payment from the user.
     * Payment is only taken when the subscription is activated. This is useful in scenarios
     * where you might want to get a user's commitment and then charge them after a certain duration or amount of usage.
     *
     * @param planHash      Unique identifying hash of the plan.
     * @param metaData      Any extra JSON data you'd like to pass/store on-chain (optional).
     * @param txData        Provide signer, gas and gasPrice information (optional).
     *
     * @returns             Unique subscription hash.
     * @priority            4
     *
    */
    SubscriptionsAPI.prototype.subscribe = function (planHash, metaData, txData) {
        return __awaiter(this, void 0, void 0, function () {
            var safeTxData, hasEnoughAllowance, balance, plan;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        safeTxData = (txData) || { from: this.web3.eth.accounts[0] };
                        return [4 /*yield*/, this.hasGivenAuthorisation(safeTxData.from || this.web3.eth.accounts[0])];
                    case 1:
                        hasEnoughAllowance = _a.sent();
                        if (!hasEnoughAllowance) {
                            throw ("EighEx does not have sufficient allowance to subscribe to take funds from a user's wallet");
                        }
                        return [4 /*yield*/, this.tokenWrapper.balanceOf(this.addressBook.transactingTokenAddress || '', safeTxData.from || this.web3.eth.accounts[0])];
                    case 2:
                        balance = _a.sent();
                        return [4 /*yield*/, this.volumeSubscriptionWrapper.getPlan(planHash)];
                    case 3:
                        plan = _a.sent();
                        if (balance.toNumber() < plan.amount.toNumber()) {
                            throw ("The user does not have enough funds in their wallet");
                        }
                        if (plan.owner == (safeTxData.from || this.web3.eth.accounts[0])) {
                            throw ("You cannot subscribe to your own plan");
                        }
                        return [2 /*return*/, this.volumeSubscriptionWrapper.createSubscription(planHash, metaData, txData)];
                }
            });
        });
    };
    /**
     * Activate
     *
     * Makes first payment. This function can only be called once a user has subscribed to a subscription plan.
     * Provided that a user has subscribed and given authorisation to 8x, the activate function can be called
     * by any user. We're still exploring the best way to lock this down while still providing enough flexibility.
     *
     *
     * @param subscriptionHash    Unique subscription hash returned upon subscribing.
     * @param txData              Provide signer, gas and gasPrice information (optional).
     *
     * @returns                   Hash of the transaction upon completion.
     * @priority                  5
     *
    */
    SubscriptionsAPI.prototype.activate = function (subscriptionHash, txData) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.executorWrapper.activateSubscription(this.addressBook.volumeSubscriptionAddress, subscriptionHash, txData)];
            });
        });
    };
    /**
     * Get subscription
     *
     * Get a subscription using the subscription hash.
     *
     * @param subscriptionHash  Unique subscription hash returned upon subscribing.
     *
     * @response
     * ```
     * {
     *    owner: '0xdfLqn83...',
     *    tokenAddress: '0xw9s3mfd...',
     *    planHash: '0xwjw93m3sd...',
     *    lastPaymentDate: 155321434, // (epoch in seconds)
     *    terminationDate: 155452222, // (epoch in seconds)
     *    subscriptionHash: '0xhad93d...',
     * }
     * ```
     *
     * @returns                 A subscription object.
     * @priority                6
     *
    */
    SubscriptionsAPI.prototype.get = function (subscriptionHash) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.volumeSubscriptionWrapper.getSubscription(subscriptionHash)];
            });
        });
    };
    /**
     * Get status
     *
     * Determine the current state of a subscription. It can either be 'active', 'processing' or 'inactive'.
     *
     * The processing state is a simply divisor of the interval of your subscription. Suppose the interval divisor is set at 7, then a 28-day subscription has a maximum of 4 days (plus an hour) to be processed.
     *
     * The extra hour is counted in the case a service node doesn't process a subscription and another node on the network can steal their tokens and process your subscription instead.
     *
     * When you call this function, we'll also let you know whether the user has enough tokens to pay for the subscription and if they've revoked authorisation for 8x to take tokens from their wallet.
     *
     * You can decide how you'd like to handle unprocessed subscriptions.
     *
     * @response
     * ```
     * // In the following response, we see the status of a user who's subscription is being processed but has a low chance of not fulfilling their recurring obligation.
     * [
     *    'processing', // The subscription is being processed
     *    'true', // The user has enough tokens to pay for the subscription
     *    'true', // The user has given 8x enough allowance to take tokens from their wallet
     * ]
     * ```
     *
     * @param subscriptionHash        Unique subscription hash returned upon subscribing.
     * @returns                       Current status, does user has enough tokens, does 8x got authorisation.
     * @priority                      7
     *
     */
    SubscriptionsAPI.prototype.getStatus = function (subscriptionHash) {
        return __awaiter(this, void 0, void 0, function () {
            var subscription, planHash, plan, intervalDivisor, now, dueDate, userBalance, hasEnough, allowanceAmount, permissionGranted;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.get(subscriptionHash)];
                    case 1:
                        subscription = _a.sent();
                        planHash = subscription.planHash;
                        return [4 /*yield*/, this.volumeSubscriptionWrapper.getPlan(planHash)];
                    case 2:
                        plan = _a.sent();
                        return [4 /*yield*/, this.executorWrapper.getIntervalDivisor()];
                    case 3:
                        intervalDivisor = _a.sent();
                        now = Date.now() / 1000;
                        dueDate = subscription.lastPaymentDate + plan.interval;
                        return [4 /*yield*/, this.tokenWrapper.balanceOf(plan.tokenAddress, subscription.owner)];
                    case 4:
                        userBalance = _a.sent();
                        hasEnough = userBalance >= plan.amount;
                        return [4 /*yield*/, this.tokenWrapper.allowance(plan.tokenAddress, subscription.owner, this.addressBook.transferProxyAddress || '')];
                    case 5:
                        allowanceAmount = _a.sent();
                        permissionGranted = allowanceAmount >= plan.amount;
                        // If we're before the payment date, happy days.
                        console.log(dueDate);
                        console.log(now);
                        console.log(subscription);
                        if (now < dueDate && subscription.terminationDate === 0) {
                            return [2 /*return*/, ['active', hasEnough, permissionGranted]];
                        }
                        // The subscription is being processed, plus give an extra hour for someone to catch out
                        if (now < dueDate + (plan.interval / intervalDivisor.toNumber()) + (60 * 60) && subscription.terminationDate === 0) {
                            return [2 /*return*/, ['processing', hasEnough, permissionGranted]];
                        }
                        return [2 /*return*/, ['inactive', hasEnough, permissionGranted]];
                }
            });
        });
    };
    /**
     * All subscriptions
     *
     * Get all the subscriptions a user has subscribed to..
     *
     * @param user       The user you'd like to get subscriptions for.
     *
     * @returns          An array of subscription objects.
     * @priority         8
     *
    */
    SubscriptionsAPI.prototype.getSubscribed = function (user) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.volumeSubscriptionWrapper.getSubscriptionsByUser(user)];
            });
        });
    };
    /**
     * Cancel a subscription
     *
     * Cancels a subscription and prevents payments from being taken next billing cycle.
     * This can only be called by the user who created the subscription in the first place.
     *
     * @param subscriptionHash    Unique subscription hash returned upon subscribing.
     * @param txData              Provide signer, gas and gasPrice information (optional).
     *
     * @returns                   Hash of the transaction upon completion.
     * @priority                  9
     *
     */
    SubscriptionsAPI.prototype.cancel = function (subscriptionHash, txData) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.volumeSubscriptionWrapper.cancelPayment(subscriptionHash, txData)];
            });
        });
    };
    return SubscriptionsAPI;
}());
exports.default = SubscriptionsAPI;
//# sourceMappingURL=subscriptions_api.js.map