import * as Web3 from 'web3';
import { TransactionReceipt } from 'ethereum-types';
import { TxHash } from '@8xprotocol/types';
export declare const BlockchainAPIErrors: {
    AWAIT_MINE_TX_TIMED_OUT: (txHash: string) => string;
};
export declare const DEFAULT_TIMEOUT_FOR_TX_MINED = 60000;
/**
  * Blockchain
  *
  * @comment The blockchain API provides a convenient way to interact with the blockchain, such as waiting for a transaction to be processed.
  *
  * @path eightEx.blockchain
  *
*/
export declare class BlockchainAPI {
    private web3;
    private intervalManager;
    private web3Utils;
    constructor(web3?: Web3);
    /**
      * Wait for transaction
      *
      * This function will wait until a transaction has been mined on the blockchain. Especially
      * useuful for providing end users with feedback on the status of their transaction.
      *
      * @param txHash                 Hash of the transaction
      * @param pollingIntervalMs      How often would you like the blockchain to be polled. Default value is set at 1,000ms (1s)
      * @param timeoutMs              After how long should the operation timeout.  Default value is set at 60,000ms (60s).
      *
      * @priority            1
      *
    */
    awaitTransactionMinedAsync(txHash: TxHash, pollingIntervalMs?: number, timeoutMs?: number): Promise<TransactionReceipt>;
}
