import * as Web3 from 'web3';
import * as ABIDecoder from 'abi-decoder';
import { TransactionReceipt } from 'ethereum-types';
import { Log } from '@8xprotocol/types';
import { BaseContract } from '@8xprotocol/artifacts/node_modules/@8xprotocol/base-contract';
export interface GetEventOptions {
    fromBlock?: number;
    toBlock?: number;
    limit?: number;
}
export declare function getFormattedLogsFromTxHash(web3: Web3, abi: any, txHash: string): Promise<Log[]>;
export declare function getFormattedLogsFromReceipt(receipt: TransactionReceipt): Log[];
/**
 * Converts a ABI Decoded Log into a Log
 * Input Example
 * {
 *   name: 'Transfer',
 *   events: [
 *    { name: 'from',
 *      type: 'address',
 *      value: '0xc604980c49f5c3be6e7e42526ec00f211e333385' },
 *    { name: 'to',
 *      type: 'address',
 *      value: '0xf8600afbf76236454a53e8dcc4d1feaa26fe1a77' },
 *    { name: 'value', type: 'uint256', value: '10000000000000000000' },
 *   ],
 *   address: '0xea76972f7587c27887aa403d84671717f6826f62'
 * }
 *
 * Output Example
 * {
 *   event: "Transfer",
 *   address: tokenAddress,
 *   args: {
 *     from,
 *     to,
 *     value,
 *   },
 * };
 */
export declare function formatLogEntry(logs: ABIDecoder.DecodedLog): Log;
export declare function getPastLogs(web3: Web3, contractWrapper: BaseContract, eventName: string, options?: GetEventOptions): Promise<ABIDecoder.DecodedLog[]>;
